package com.goats.mqtt;

import com.goats.mapper.MqttDataMapper;
import com.goats.pojo.DataPacket;
import com.goats.pojo.MqttData;
import net.sf.json.JSONObject;
import org.dromara.mica.mqtt.codec.MqttPublishMessage;
import org.dromara.mica.mqtt.core.client.IMqttClientGlobalMessageListener;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Service;
import org.tio.core.ChannelContext;

import java.nio.charset.StandardCharsets;
import java.util.Iterator;
import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;

@Service
public class MqttClientGlobalMessageListener implements IMqttClientGlobalMessageListener {

    private static final Logger logger = LoggerFactory.getLogger(MqttClientGlobalMessageListener.class);

    @Autowired
    private MqttDataMapper mqttDataMapper;
    // 超时时间，单位为毫秒
    private static final long TIMEOUT = 20000; // 20秒

    // 模拟接收到的数据包 Map，使用设备ID和数据时间戳组合来唯一标识
    //key 设备主键 value 三个包信息
    private static ConcurrentHashMap<String, DataPacket> messageMap = new ConcurrentHashMap<>();

    @Override
    public void onMessage(ChannelContext context, String topic, MqttPublishMessage message, byte[] payload) {
        // 在这里处理收到的消息
        try {
            String str = new String(payload, StandardCharsets.UTF_8);
            JSONObject jsonObject = new JSONObject(str);
            logger.info("收到的消息订阅是{}", topic);
            String deviceKey = jsonObject.optString("DeviceID"); //设备Id
            String deviceDate = jsonObject.optString("Date"); //网关时间
            Integer seq = jsonObject.optInt("Seq"); //包编号
            if (topic.equals("v1/devices")&& deviceKey.equals("1016")) {
                /**
                 * 1、根据每条发送过来的消息，存储的Map集合中（选择合适的key 比如设备id 时间戳不同不能作为联合主键）
                 * 2、依次接收1、2、3存入，每次接收到消息判断是否全部接收到3条消息，如果已全部接收到，执行下面逻辑，插入数据库
                 * 3、如果包顺序是按照老狗网关配置顺序（不管按不按顺序），一旦出现重复的SEQ消息，那么直接丢弃上一个对象，重新存储map集合。
                 *
                 * 改造:
                 *      1、超时处理（只收到了 2 个包，第 3 个包丢失了、缓存中的数据会一直存在）
                 *      2、重复包的处理逻辑 是丢失上个整个对象，还是丢弃上次的包，还是丢弃本次的包。==== 丢弃本次发来的包
                 */
                //从map缓存对象中判断是否存在包信息
                DataPacket existingPacket = messageMap.get(deviceKey);
                //如果不存在包信息，那么是第一条信息，创建包对象
                if (existingPacket == null) {
                    //添加包编号与JSON数据
                    existingPacket = new DataPacket(deviceKey, System.currentTimeMillis(), seq, str);
                } else {
                    // 如果存在包信息，检查是否重复
                    if (existingPacket.isSeqReceived(seq)) {
                        // 重复包，丢弃当前包
                        System.out.println("重复包，设备ID: " + deviceKey + ", 包编号: " + seq);
                        return;
                    } else {
                        // 添加新包
                        existingPacket.addPacket(seq, str);
                    }
                }
                //判断如何缓存中已存在三个包消息，那么进行业务逻辑处理
                if (existingPacket.isComplete()) {
                    //合并三个JSON串
                    String mergeJsonStrings = existingPacket.getMergedData();
                    logger.info("设备ID是->{},合并后JSON串->{}", deviceKey, mergeJsonStrings);
                    String DeviceID = jsonObject.optString("DeviceID"); //设备Id
                    messageMap.remove(DeviceID);  // 数据保存后，移除此数据包记录
                    logger.info("-------------------执行业务逻辑在下面-------------------------");
                    jsonObject = new JSONObject(mergeJsonStrings);
                    //数据添加到数据库
                    MqttData mqttData = new MqttData();
                    mqttData.setRid(String.valueOf(System.currentTimeMillis()));
                    mqttData.setId(jsonObject.optString("DeviceID"));
                    mqttData.setA001(jsonObject.optDouble("A001"));
                    mqttData.setA002(jsonObject.optDouble("A002"));
                    mqttData.setA003(jsonObject.optDouble("A003"));
                    mqttData.setA004(jsonObject.optInt("A004"));
                    mqttData.setA005(jsonObject.optInt("A005"));
                    mqttData.setA006(jsonObject.optInt("A006"));
                    mqttData.setA007(jsonObject.optInt("A007"));
                    mqttData.setA008(jsonObject.optInt("A008"));
                    mqttData.setA009(jsonObject.optInt("A009"));
                    mqttData.setA010(jsonObject.optInt("A010"));
                    mqttData.setA011(jsonObject.optInt("A011"));
                    mqttData.setA012(jsonObject.optInt("A012"));
                    mqttData.setA013(jsonObject.optInt("A013"));
                    mqttData.setA014(jsonObject.optInt("A014"));
                    mqttData.setA015(jsonObject.optInt("A015"));
                    mqttData.setA016(jsonObject.optInt("A016"));
                    mqttData.setA017(jsonObject.optInt("A017"));
                    mqttData.setA018(jsonObject.optDouble("A018"));
                    mqttData.setA019(jsonObject.optDouble("A019"));
                    mqttData.setA020(jsonObject.optDouble("A020"));
                    mqttData.setA021(jsonObject.optInt("A021"));
                    mqttData.setA022(jsonObject.optInt("A022"));
                    mqttData.setA023(jsonObject.optInt("A023"));
                    mqttData.setA024(jsonObject.optInt("A024"));
                    mqttData.setA025(jsonObject.optInt("A025"));
                    mqttData.setA026(jsonObject.optInt("A026"));
                    mqttData.setA027(jsonObject.optInt("A027"));
                    mqttData.setA028(jsonObject.optDouble("A028"));
                    mqttData.setA029(jsonObject.optDouble("A029"));
                    mqttData.setA030(jsonObject.optDouble("A030"));
                    mqttData.setA031(jsonObject.optInt("A031"));
                    mqttData.setA032(jsonObject.optInt("A032"));
                    mqttData.setA033(jsonObject.optInt("A033"));
                    mqttData.setA034(jsonObject.optInt("A034"));
                    mqttData.setA035(jsonObject.optInt("A035"));
                    mqttData.setA036(jsonObject.optInt("A036"));
                    mqttData.setA037(jsonObject.optInt("A037"));
                    mqttData.setA038(jsonObject.optInt("A038"));
                    mqttData.setA039(jsonObject.optString("A039"));
                    mqttData.setA040(jsonObject.optInt("A040"));
                    mqttData.setA041(jsonObject.optInt("A041"));
                    mqttData.setA042(jsonObject.optInt("A042"));
                    mqttData.setA043(jsonObject.optInt("A043"));
                    mqttData.setA044(jsonObject.optInt("A044"));
                    mqttData.setA045(jsonObject.optInt("A045"));
                    mqttData.setA046(jsonObject.optInt("A046"));
                    mqttData.setA047(jsonObject.optInt("A047"));
                    mqttData.setA048(jsonObject.optDouble("A048"));
                    mqttData.setA049(jsonObject.optDouble("A049"));
                    mqttData.setA050(jsonObject.optDouble("A050"));
                    mqttData.setA051(jsonObject.optDouble("A051"));
                    mqttData.setA052(jsonObject.optDouble("A052"));
                    mqttData.setA053(jsonObject.optDouble("A053"));
                    mqttData.setA054(jsonObject.optDouble("A054"));
                    mqttData.setA055(jsonObject.optDouble("A055"));
                    mqttData.setA056(jsonObject.optDouble("A056"));
                    mqttData.setA057(jsonObject.optDouble("A057"));
                    mqttData.setA058(jsonObject.optDouble("A058"));
                    mqttData.setA059(jsonObject.optDouble("A059"));
                    mqttData.setA060(jsonObject.optDouble("A060"));
                    mqttData.setA061(jsonObject.optDouble("A061"));
                    mqttData.setA062(jsonObject.optDouble("A062"));
                    mqttData.setA063(jsonObject.optDouble("A063"));
                    mqttData.setA064(jsonObject.optDouble("A064"));
                    mqttData.setA065(jsonObject.optDouble("A065"));
                    mqttData.setA066(jsonObject.optDouble("A066"));
                    mqttData.setA067(jsonObject.optDouble("A067"));
                    mqttData.setA068(jsonObject.optDouble("A068"));
                    mqttData.setA069(jsonObject.optDouble("A069"));
                    mqttData.setA070(jsonObject.optDouble("A070"));
                    mqttData.setA071(jsonObject.optDouble("A071"));
                    mqttData.setA072(jsonObject.optDouble("A072"));
                    mqttData.setA073(jsonObject.optDouble("A073"));
                    mqttData.setA074(jsonObject.optDouble("A074"));
                    mqttData.setA075(jsonObject.optDouble("A075"));
                    mqttData.setA076(jsonObject.optDouble("A076"));
                    mqttData.setA077(jsonObject.optDouble("A077"));
                    mqttData.setA078(jsonObject.optDouble("A078"));
                    mqttData.setA079(jsonObject.optDouble("A079"));
                    mqttData.setA080(jsonObject.optDouble("A080"));
                    mqttData.setA081(jsonObject.optInt("A081"));
                    mqttData.setA082(jsonObject.optInt("A082"));
                    mqttData.setA083(jsonObject.optInt("A083"));
                    mqttData.setA084(jsonObject.optInt("A084"));
                    mqttData.setA085(jsonObject.optInt("A085"));
                    mqttDataMapper.insertMqttData(mqttData);
                } else {
                    logger.info("设备ID是->{},不满足三个包信息,进行PUT操作,保持对象信息是{}", deviceKey, existingPacket);
                    messageMap.put(deviceKey, existingPacket);
                    System.out.println("messageMap = " + messageMap);
                    return;
                }

            }else {
                logger.info("次信息是1015设备，不做处理，{}",jsonObject);
            }
        } catch (Exception e) {
            e.getMessage();
        }
    }


    @Scheduled(fixedRate = 5000)
    public void checkTimeout() {
        long currentTime = System.currentTimeMillis();
        logger.info("定时任务正在执行,{}",messageMap);
        for (Iterator<Map.Entry<String, DataPacket>> it = messageMap.entrySet().iterator(); it.hasNext(); ) {
            Map.Entry<String, DataPacket> entry = it.next();
            DataPacket packet = entry.getValue();
            long timeDiff = currentTime - packet.getLastUpdateTime();
            logger.info("定时任务正在执行，设备ID: {}, 时间差: {} 毫秒", packet.getDeviceId(), timeDiff);
            if (timeDiff > TIMEOUT) {
                it.remove();
                logger.info("定时任务正在执行，超时移除，设备ID: {}", packet.getDeviceId());
            }
        }
    }

}
